/*****************************************************************************/
/*		This sample demonstrates how to acquire a  color picture in		     */
/*      stillcolor mode using high level snap function			     		 */
/*																			 */
/*		Contrast, Saturation and brightness can be changed using attributes  */
/*		see Function Rference Manual.										 */
/*																			 */
/*		The display is done using the function imgPlot, the canvas object	 */																	
/*		is used to reserve some room on the panel.							 */
/*																			 */
/*		It is also possible to display the picture using a Canvas or a Graph */
/*		See the Display sub-directory										 */
/*****************************************************************************/    


#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>   

#define _NIWIN  

#include "niimaq.h"
#include "Basic StillColor.h"


// globals
static int panelHandle;
static SESSION_ID Sid;
static INTERFACE_ID Iid;
static Int8* ImaqBuffer=NULL;
static unsigned int CanvasWidth, CanvasHeight, CanvasTop, CanvasLeft;
static int CVIWndHndl;


int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "Basic StillColor.uir", PANEL)) < 0)
		return -1;
	
	// Get the Canvas size and position
	GetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_TOP, &CanvasTop);
	GetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_LEFT, &CanvasLeft);        
	GetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_WIDTH, &CanvasWidth);        
	GetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_HEIGHT, &CanvasHeight); 
	
	// Get the Panel Window Handle to display with imgPlot
	GetPanelAttribute (panelHandle, ATTR_SYSTEM_WINDOW_HANDLE, &CVIWndHndl);
	
		
	DisplayPanel (panelHandle);
	RunUserInterface ();
	
	return 0;
}



int CVICALLBACK Quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
	}
	return 0;
}



int CVICALLBACK Snap (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int 			acqWinWidth, acqWinHeight;
	unsigned long	plotFlag, colorMode, imageRep;
	char			intfName[64];      
	
	switch (event)
	{
		case EVENT_COMMIT:
			// Get the interface name
			GetCtrlVal (panelHandle, PANEL_INTFNAME, intfName);
		
			// Open an interface and a session
			imgInterfaceOpen (intfName, &Iid);
			imgSessionOpen (Iid, &Sid);
			
			// Set the Color Mode and the Output format    
			GetCtrlVal (panelHandle, PANEL_COLORMODE, &colorMode);
			GetCtrlVal (panelHandle, PANEL_IMAGEREP, &imageRep);
			imgSetAttribute (Sid, IMG_ATTR_COLOR_MODE, colorMode);
			
			if(colorMode != IMG_COLOR_MODE_DISABLED)
				imgSetAttribute (Sid, IMG_ATTR_COLOR_IMAGE_REP, imageRep);
			else
				imageRep = IMG_COLOR_REP_RED8; // any 8 bits representation
			
			// select the display mode according to the image representation
			switch (imageRep)
			{
				case IMG_COLOR_REP_RGB32:
					plotFlag = IMGPLOT_COLOR_RGB32;
					break;
				case IMG_COLOR_REP_RED8:
				case IMG_COLOR_REP_GREEN8:
				case IMG_COLOR_REP_BLUE8:
				case IMG_COLOR_REP_LUM8:
				case IMG_COLOR_REP_HUE8:
				case IMG_COLOR_REP_SAT8:  
				case IMG_COLOR_REP_INT8:
					plotFlag = IMGPLOT_MONO_8;
					break;
				case IMG_COLOR_REP_LUM16: 
				case IMG_COLOR_REP_HUE16:    
				case IMG_COLOR_REP_SAT16:
				case IMG_COLOR_REP_INT16:
					plotFlag = IMGPLOT_MONO_16;
					break;
				case IMG_COLOR_REP_RGB48:
					plotFlag = 0xFFFFFFFF;
					break;
				case IMG_COLOR_REP_RGB24:
					plotFlag = IMGPLOT_COLOR_RGB24;
					break;
				case IMG_COLOR_REP_RGB16:
					plotFlag = 0xFFFFFFFF;
					break;
				case IMG_COLOR_REP_HSL32:     
				case IMG_COLOR_REP_HSI32:
					plotFlag = IMGPLOT_COLOR_RGB32;
					break;
			}
		
			// Let's check that the Acquisition window is not smaller than the Canvas
			imgGetAttribute (Sid, IMG_ATTR_ACQWINDOW_WIDTH, &acqWinWidth);
			imgGetAttribute (Sid, IMG_ATTR_ACQWINDOW_HEIGHT, &acqWinHeight);
			
			if(CanvasWidth > acqWinWidth)
				CanvasWidth = acqWinWidth;
			if(CanvasHeight > acqWinHeight)
				CanvasHeight = acqWinHeight;
				
			// Set the ROI and the rowbytes to the size of the Canvas so that it will fit nicely
			imgSetAttribute (Sid, IMG_ATTR_ROWPIXELS, CanvasWidth); 
			imgSetAttribute (Sid, IMG_ATTR_ROI_WIDTH, CanvasWidth);
			imgSetAttribute (Sid, IMG_ATTR_ROI_HEIGHT, CanvasHeight);
			
			
			// snap a picture : ImaqBuffer is NULL, memory will be managed by NI-IMAQ
			ImaqBuffer = NULL;
			imgSnap (Sid, (void **)&ImaqBuffer);
			
			
			// Display using NI-IMAQ
			if (plotFlag != 0xFFFFFFFF)
				imgPlot (CVIWndHndl, ImaqBuffer, 0, 0, CanvasWidth, CanvasHeight,
							   CanvasLeft, CanvasTop, plotFlag);
			else
				MessagePopup ("Warning",
							  "This Image representation is not supported by the display function : imgPlot");
			
			// Close the interface and the session
			imgClose (Sid, TRUE);
			imgClose (Iid, TRUE);
			break;
	}
	return 0;
}


